/*
 * Segment 2 (KEx),  SWE 2.1 LMS
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lms.empfang.ftpserver;

import java.io.File;
import java.io.FilenameFilter;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import de.bsvrz.kex.lms.vew.LMSFTPListener;
import de.bsvrz.kex.lms.vew.LMSKommunikationsstatus;
import de.bsvrz.kex.lms.vew.LMSParameter;
import de.bsvrz.kex.lms.vew.LMSProperties;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Modul FTP-Server.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 */
public class LMSFTPServer extends Thread {
	/**
	 * Filter zur Dateiauswahl.
	 */
	class XmlFilenameFilter implements FilenameFilter {
		@Override
		public boolean accept(final File f, final String s) {
			return s.toLowerCase().endsWith(".xml");
		}
	}

	/**
	 * DebugLogger für Debug-Ausgaben.
	 */
	private final Debug debug = Debug.getLogger();

	/**
	 * Zyklus, mit dem das Eingangsverzeichnis auf neue Dateien
	 * &uuml;berpr&uuml;ft wird.
	 */
	private final int zyklus = 10000;

	/**
	 * Die aktuellen Parameter.
	 */
	private LMSParameter parameter = null;

	/**
	 * Liste der angemeldeten Listener.
	 */
	private final List<LMSFTPListener> listenerList = new ArrayList<LMSFTPListener>();

	/**
	 * Der Kommunikationsstatus mit der Landesmeldestelle.
	 */
	private LMSKommunikationsstatus kommunikationsStatus = LMSKommunikationsstatus.GESTOERT;

	/**
	 * HashSet der bereits empfangenen Dateien.
	 */
	private HashSet<File> dateimap = new HashSet<File>();

	/**
	 * Zeitpunkt des letzten Empfanges einer Datei.
	 */
	private long letzterEmpfang = 0;

	/**
	 * Run-Flag.
	 */
	boolean run = true;

	/**
	 * Erzeugt eine Instanz und startet diese als Thread.
	 *
	 * @param parameter
	 *            Der Parametersatz
	 */
	public LMSFTPServer(final LMSParameter parameter) {
		this.parameter = parameter;
		this.setName(this.getClass().getSimpleName());
		// selbst starten
		start();
	}

	/**
	 * F&uuml;gt einen Listener hinzu.
	 *
	 * @param listner
	 *            neuer Listener
	 */
	public void addListener(final LMSFTPListener listner) {
		listenerList.add(listner);
	}

	/**
	 * &Uuml;berpr&uuml;ft das Eingangsverzeichnis auf neue empfangene Dateien.
	 */
	private void findeNeueDateien() {
		final HashSet<File> dateimapneu = new HashSet<File>();

		final File dir = new File(parameter.getEingangsverzeichnis());

		final File[] dateien = dir.listFiles(new XmlFilenameFilter());

		final List<File> neueDateien = new ArrayList<File>();

		if (dateien != null) {
			for (final File datei : dateien) {
				if (datei.getName().equalsIgnoreCase(LMSProperties.KEEP_ALIVE_DATEINAME)) {
					debug.info("KeepAlive-Datei empfangen");
					letzterEmpfang = System.currentTimeMillis();
					setzeKommunikationsStatus(LMSKommunikationsstatus.NICHT_GESTOERT);
					datei.delete();
					continue;
				}

				dateimapneu.add(datei);
				// bereits empfangene ignorieren
				if (!dateimap.contains(datei)) {
					neueDateien.add(datei);
				}
			}
		}

		dateimap = dateimapneu;

		if (neueDateien.size() > 0) {
			debug.fine(neueDateien.size() + " neue Datei(en) empfangen (" + neueDateien + ")");
			setzeKommunikationsStatus(LMSKommunikationsstatus.NICHT_GESTOERT);
			letzterEmpfang = System.currentTimeMillis();
			informiereListenerDateien(neueDateien);
		}
	}

	/**
	 * Informiert die Listener &uuml;ber neue Dateien.
	 *
	 * @param dateien
	 *            Liste der neuen Dateien
	 */
	void informiereListenerDateien(final List<File> dateien) {
		for (final LMSFTPListener listener : listenerList) {
			listener.empfangen(dateien);
		}
	}

	/**
	 * Informiert die Listener &uuml;ber den Kommunikationsstatus.
	 */
	void informiereListenerStatus() {
		for (final LMSFTPListener listener : listenerList) {
			listener.kommunikationsStatus(kommunikationsStatus);
		}
	}

	@Override
	public void run() {
		try {
			while (run) {
				Thread.sleep(zyklus);
				findeNeueDateien();

				// ueberpruefe Kommunikationsstatus
				if ((System.currentTimeMillis() - letzterEmpfang) > parameter.getTestIntervallKommunikation()) {
					debug.warning("Der Kommunikationsstatus ist gestört");
					setzeKommunikationsStatus(LMSKommunikationsstatus.GESTOERT);
					letzterEmpfang = System.currentTimeMillis();
				}
			}
		} catch (final InterruptedException ie) {
			/**
			 *
			 */
		} catch (final Exception e) {
			debug.error(e.getMessage());
		}

	}

	/**
	 * Setzt den internen Kommunikationstatus und benachrichtigt die Listener.
	 *
	 * @param neuerStatus
	 *            neuer Kommunikationsstatus
	 */
	private void setzeKommunikationsStatus(final LMSKommunikationsstatus neuerStatus) {
		final boolean notify = neuerStatus != kommunikationsStatus;
		kommunikationsStatus = neuerStatus;
		if (notify) {
			informiereListenerStatus();
		}
	}

	/**
	 * Stoppt den Thread.
	 */
	public void stopp() {
		this.run = false;
		this.interrupt();
	}
}
