/*
 * Segment 2 (KEx),  SWE 2.1 LMS
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lms.vew.daten;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.kex.lms.vew.RDSException;

/**
 * Abstrakte Implementation eines Informationselementes einer RDS-Meldung.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 */

public abstract class AbstractRDSInfoElement implements RDSInfoElement {

	/**
	 * Der zum Element geh&ouml;rende Name des XML-Elementes.
	 */
	protected String xmlElementName = "";

	/**
	 * Der Name des zum Element geh&ouml;renden DAV-Attributes.
	 */
	protected String davAttributName = "";

	/**
	 * Liste der Kind-Elemente.
	 */
	List<AbstractRDSInfoElement> kinder = new ArrayList<AbstractRDSInfoElement>();

	/**
	 * Erzeugt ein Element.
	 *
	 * @param xmlName
	 *            Name des XML-ELementes
	 * @param davName
	 *            Name des DAV-Attributes
	 */
	public AbstractRDSInfoElement(final String xmlName, final String davName) {
		super();
		this.xmlElementName = xmlName;
		this.davAttributName = davName;
	}

	@Override
	public void aktualisieren(final Data davdata) throws RDSException {
		getDavDaten(davdata);

		Data ddata;
		if ((getDavAttributName() != null) && (getDavAttributName().length() > 0)) {
			ddata = davdata.getItem(getDavAttributName());
		} else {
			ddata = davdata;
		}

		for (final RDSInfoElement child : kinder) {
			child.aktualisieren(ddata);
		}
	}

	/**
	 * F&uuml;gt ein Kind-Element hinzu.
	 *
	 * @param child
	 *            Kind-Element
	 */
	public void appendChild(final AbstractRDSInfoElement child) {
		kinder.add(child);
	}

	@Override
	public void einlesenXML(final Node node) throws RDSException {
		getXmlDaten(node);

		if (kinder.size() > 0) {
			final NodeList nl = node.getChildNodes();
			for (int i = 0; i < nl.getLength(); i++) {
				final Node n = nl.item(i);
				for (final RDSInfoElement child : kinder) {
					if (child.hatNodeHandler(n)) {
						child.einlesenXML(n);
					}
				}
			}
		}
	}

	@Override
	public Element erzeugeDocumentElement(final Document doc, final Element parent) throws RDSException {
		final Element element = doc.createElement(getXmlElementName());
		parent.appendChild(element);
		setXmlDaten(element);

		for (final RDSInfoElement child : kinder) {
			child.erzeugeDocumentElement(doc, element);
		}

		return element;
	}

	/**
	 * Gibt den zum Element geh&ouml;renden Namen des DAV-Attributes
	 * zur&uuml;ck.
	 *
	 * @return Name des DAV-Attributes
	 */
	protected String getDavAttributName() {
		return davAttributName;
	}

	@Override
	public void getDavDaten(final Data davdata) throws RDSException {
		/** keine eigenen Daten */
	}

	@Override
	public void getXmlDaten(final Node node) throws RDSException {
		/** keine eigenen Daten */
	}

	/**
	 * Gibt den zum Element geh&ouml;rende XML-Element Namen zur&uuml;ck.
	 *
	 * @return XML-Element Name
	 */
	protected String getXmlElementName() {
		return xmlElementName;
	}

	/**
	 * Test, ob das Objekt einen Handler f&uuml;r einen XML-Dokument-Knoten
	 * besitzt.
	 *
	 * @param node
	 *            ein XML-Dokument-Knoten
	 * @return true, wenn das Objekt einen Handler f&uuml;r einen
	 *         XML-Dokument-Knoten besitzt, sonst false
	 */
	@Override
	public boolean hatNodeHandler(final Node node) {
		return node.getNodeName().equals(xmlElementName);
	}

	@Override
	public void publizieren(final Data davdata) throws RDSException {
		setDavDaten(davdata);

		Data ddata;
		if ((getDavAttributName() != null) && (getDavAttributName().length() > 0)) {
			ddata = davdata.getItem(getDavAttributName());
		} else {
			ddata = davdata;
		}

		for (final RDSInfoElement child : kinder) {
			child.publizieren(ddata);
		}
	}

	@Override
	public void setDavDaten(final Data davdata) throws RDSException {
		/** keine eigenen Daten */
	}

	@Override
	public void setXmlDaten(final Element element) throws RDSException {
		/** keine eigenen Daten */
	}

}
