/*
 * Segment 2 (KEx),  SWE 2.1 LMS
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lms.vew.daten;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.kex.lms.vew.RDSException;

/**
 * Informationen zu den Locationen.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 */

public class RDSLocationDaten extends AbstractRDSInfoElement {

	/**
	 * TIC Info XML Element: Location, enth&auml;lt alle Informationen zur den
	 * Locationen.
	 */
	private static final String XML_ELEMENT_LOCATION = "LCA";

	/**
	 * Location-Format. TIC Info XML Element: LocationFormat, Format des
	 * referenzierten Location-Systems.
	 */
	private static final String XML_ELEMENT_LOCATIONFORMAT = "LFM";

	/**
	 * Ausdehnung der prim&auml;ren Location TIC Info XML Element:
	 * LocationExtent, Ausdehnung der prim&auml;ren Location, entspricht der
	 * Schrittweite innerhalb der LCL ausgehend von der prim&auml;ren Location.
	 */
	private static final String XML_ELEMENT_LOCATIONEXTEND = "LEX";

	/**
	 * Location-Kodierung TIC Info XML Element: LocationMethod, die
	 * Kodierungsmethode der Location.
	 */
	private static final String XML_ELEMENT_LOCATIONMETHOD = "LME";

	/**
	 * TMC-Richtung TIC Info XML Element: LocationDirection, bezeichnet die
	 * TMC-Richtung der Location.
	 */
	private static final String XML_ELEMENT_LOCATIONDIRECTION = "LDR";

	/**
	 * Entfernung zur prim&auml;ren Location in m TIC Info XML Element:
	 * LocationPrimaryDistance.
	 */
	private static final String XML_ELEMENT_LOCATIONPRIMARYDISTANCE = "LPD";

	/**
	 * Entfernung zur prim&auml;ren Location in m TIC Info XML Element:
	 * LocationSecondaryDistance.
	 */
	private static final String XML_ELEMENT_LOCATIONSECONDARYDISTANCE = "LSD";

	/**
	 * Locationcode Vorg&auml;nger der sekund&auml;ren Location TIC Info XML
	 * Element: LocationPreLocation.
	 */
	private static final String XML_ELEMENT_LOCATIONPRELOCATION = "LPR";

	/**
	 * Locationcode Nachfolger der prim&auml;ren Location TIC Info XML Element:
	 * LocationPostLocation.
	 */
	private static final String XML_ELEMENT_LOCATIONPOSTLOCATION = "LPO";

	/**
	 * Kategorie der Location TIC Info XML Element: LocationCategory, Kategorie
	 * der Location (nur bei LocationFormat = TIC Location).
	 */
	private static final String XML_ELEMENT_LOCATIONCATEGORY = "LCG";

	/**
	 * Text zur Beschreibung der Richtung einer Strasse oder eines Segmentes.
	 * TIC Info XML Element: LocationDirectionTextID, Zuordnung einer ID zu
	 * einem Richtungstext.
	 */
	private static final String XML_ELEMENT_LOCATIONDIRECTIONTEXTID = "LDT";

	/**
	 * Liste der Locationen.
	 */
	List<RDSLocation> rdsLocationen = new ArrayList<RDSLocation>();

	/**
	 * Erzeugt ein neues Objekt mit Standardwerten.
	 */
	public RDSLocationDaten() {
		super(RDSLocationDaten.XML_ELEMENT_LOCATION, "LocationDaten");
		appendChild(new RDSInteger(RDSLocationDaten.XML_ELEMENT_LOCATIONFORMAT, "RDSLocationFormat"));
		appendChild(new RDSInteger(RDSLocationDaten.XML_ELEMENT_LOCATIONEXTEND, "LocationExtent"));
		appendChild(new RDSInteger(RDSLocationDaten.XML_ELEMENT_LOCATIONMETHOD, "LocationMethode"));
		appendChild(new RDSInteger(RDSLocationDaten.XML_ELEMENT_LOCATIONDIRECTION, "TMCRichtung"));
		appendChild(new RDSLong(RDSLocationDaten.XML_ELEMENT_LOCATIONPRIMARYDISTANCE, "LocationPrimärEntfernung"));
		appendChild(new RDSLong(RDSLocationDaten.XML_ELEMENT_LOCATIONSECONDARYDISTANCE, "LocationSekundärEntfernung"));
		appendChild(new RDSInteger(RDSLocationDaten.XML_ELEMENT_LOCATIONPRELOCATION, "LocationVorLocation"));
		appendChild(new RDSInteger(RDSLocationDaten.XML_ELEMENT_LOCATIONPOSTLOCATION, "LocationNachLocation"));
		appendChild(new RDSZustand(RDSLocationDaten.XML_ELEMENT_LOCATIONCATEGORY, "LocationKategorie"));
		appendChild(new RDSInteger(RDSLocationDaten.XML_ELEMENT_LOCATIONDIRECTIONTEXTID, "LocationRichtungTextID"));
	}

	@Override
	public void aktualisieren(final Data davdata) throws RDSException {
		super.aktualisieren(davdata);
		final Data data = davdata.getItem(getDavAttributName());

		final Data.Array locarray = data.getArray("RDSLocation");
		final int anzahl = locarray.getLength();
		rdsLocationen.clear();

		for (int i = 0; i < anzahl; i++) {
			final RDSLocation loc = new RDSLocation();
			loc.aktualisieren(locarray.getItem(i));
			rdsLocationen.add(loc);
		}
	}

	@Override
	public void einlesenXML(final Node node) throws RDSException {
		super.einlesenXML(node);
		final NodeList nl = node.getChildNodes();
		for (int i = 0; i < nl.getLength(); i++) {
			final Node n = nl.item(i);
			if (n.getNodeName().equals(RDSLocation.XML_ELEMENT_LOCATIONLOC)) {
				final RDSLocation loc = new RDSLocation();
				loc.einlesenXML(n);
				// appendChild(loc);
				rdsLocationen.add(loc);
			}
		}
	}

	@Override
	public Element erzeugeDocumentElement(final Document doc, final Element parent) throws RDSException {
		final Element oelement = super.erzeugeDocumentElement(doc, parent);

		for (final RDSLocation location : rdsLocationen) {
			location.erzeugeDocumentElement(doc, oelement);
		}

		return oelement;
	}

	/**
	 * Gibt die Locationcodes aller Locations zur&uuml;ck.
	 *
	 * @return Liste von Locationcodes
	 */
	@SuppressWarnings("boxing")
	public List<Integer> getLocationCodes() {
		final List<Integer> locs = new ArrayList<Integer>();

		for (final RDSLocation loc : rdsLocationen) {
			locs.add(loc.getLocationCode());
		}

		return locs;
	}

	@Override
	public void publizieren(final Data davdata) throws RDSException {
		super.publizieren(davdata);

		final Data data = davdata.getItem(getDavAttributName());

		final Data.Array locarray = data.getArray("RDSLocation");
		final int anzahl = rdsLocationen.size();
		locarray.setLength(anzahl);
		for (int i = 0; i < anzahl; i++) {
			rdsLocationen.get(i).publizieren(locarray.getItem(i));
		}
	}
}
