/*
 * Segment 2 (KEx),  SWE 2.1 LMS
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lms.vew.daten;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.kex.lms.vew.RDSException;

/**
 * RDS Location-Tabelle. spezifiziert die zugrundeliegende Location-Tabelle.
 * <p>
 * Die Elemente 'LocationTableCountryCode' und 'LocationTableNumber' der
 * TIC-Spezifikation werden wie folgt auf die Tabelle nach prEN ISO 14819-3
 * Annex B abgebildet:
 * </p>
 * <p>
 * Der Wert dieses Aufz&auml;hlungstyps ist eine sechsstellige Zahl der Form
 * CCCTTT mit CCC gleich Landeskennung (Country) aus TIC-Info-Element
 * 'LocationTableCountryCode', und TTT gleich Tabelle (Table) aus
 * TIC-Info-Element 'LocationTableNumber' jeweils mit f&uuml;hrenden Nullen.
 * </p>
 * <p>
 * Entsprechend prEN ISO 14819-3 Annex B sind mehrere Tabellen pro Landeskennung
 * m&ouml;glich.
 * </p>
 * <p>
 * Beispiel für die LCL der BASt:
 * </p>
 * <p>
 * LocationTableCountryCode = 13 -&gt; CCC=013
 * </p>
 * <p>
 * LocationTableNumber = 01 -&gt; TTT = 001
 * </p>
 * <p>
 * Wert: 013001
 * </p>
 *
 * @author BitCtrl Systems GmbH, Gieseler
 */

public class RDSLocationTabelle extends RDSZustand {

	/**
	 * TIC Info XML Element: LocationTableNumber, Nummer der benutzten
	 * Location-Tabelle.
	 */
	private static final String XML_ELEMENT_LOCATIONTABLENUMBER = "LCN";

	/**
	 * TIC Info XML Element: LocationTableCountryCode, Landeskennung der
	 * benutzten Location-Tabelle.
	 */
	private static final String XML_ELEMENT_LOCATIONTABLECOUNTRYCODE = "LCC";

	/**
	 * Die Nummer der Locationtabelle.
	 */
	private int tabelleNummer = 0;

	/**
	 * Die Landeskennung der Locationtabelle.
	 */
	private int tabelleLand = 0;

	/**
	 * Konstruktor.
	 */
	public RDSLocationTabelle() {
		super(RDSLocationTabelle.XML_ELEMENT_LOCATIONTABLENUMBER, "LocationTabelle");
	}

	@Override
	public void einlesenXML(final Node node) throws RDSException {
		if (node.getNodeName().equals(RDSLocationTabelle.XML_ELEMENT_LOCATIONTABLENUMBER)) {
			tabelleNummer = Integer.parseInt(node.getTextContent());
		}

		if (node.getNodeName().equals(RDSLocationTabelle.XML_ELEMENT_LOCATIONTABLECOUNTRYCODE)) {
			tabelleLand = Integer.parseInt(node.getTextContent());
		}

		// TODO entfernen, wenn KB aktualisiert
		if (tabelleLand == 0) {
			tabelleLand = 13;
		}
		if (tabelleNummer == 0) {
			tabelleNummer = 1;
		}

		wert = (tabelleLand * 1000) + tabelleNummer;
	}

	@Override
	public Element erzeugeDocumentElement(final Document doc, final Element parent) {
		Element element = doc.createElement(RDSLocationTabelle.XML_ELEMENT_LOCATIONTABLENUMBER);
		parent.appendChild(element);
		element.setTextContent(String.valueOf(tabelleNummer));

		element = doc.createElement(RDSLocationTabelle.XML_ELEMENT_LOCATIONTABLECOUNTRYCODE);
		parent.appendChild(element);
		element.setTextContent(String.valueOf(tabelleLand));

		return element;
	}

	@Override
	public void getDavDaten(final Data davdata) throws RDSException {
		final int davwert = davdata.getUnscaledValue(getDavAttributName()).intValue();
		tabelleLand = davwert / 1000;
		tabelleNummer = davwert % 1000;
	}

	@Override
	public boolean hatNodeHandler(final Node node) {
		return node.getNodeName().equals(RDSLocationTabelle.XML_ELEMENT_LOCATIONTABLENUMBER)
				|| node.getNodeName().equals(RDSLocationTabelle.XML_ELEMENT_LOCATIONTABLECOUNTRYCODE);
	}

	@Override
	public void setDavDaten(final Data davdata) throws RDSException {
		davdata.getUnscaledValue(getDavAttributName()).set((tabelleLand * 1000) + tabelleNummer);
	}
}
